# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.shortcuts import render, redirect

from GeoInt_SIDT.settings import BASE_URL
from catalog.forms import ASFSearchForm
from catalog.models import Polygon, CartProduct, Product_l1c, Search, Purchase, Process
from django.http import HttpResponse, JsonResponse, HttpResponseRedirect
from django.urls import reverse
from django.views.generic.base import View
from django.template import loader, RequestContext
from django.contrib.auth.models import User
import requests, json, unicodedata, sys, base64
import os
from geosentinel import APISentinel, polygonToBox
from mail.views import wsMail, wsMail2
from django.contrib import messages

dirname = os.path.dirname(__file__)
configfile = os.path.join(dirname, '../config/config.json')

with open(configfile, 'r') as f:
    config = json.load(f)

NAS_PATH = config['PATHS']['PATH_NAS']
USERS_PATH = config['PATHS']['PATH_USERS']
SENTINEL_PATH = config['PATHS']['PATH_GEOSENTINEL']
API_SENTINEL_USER = config['API_SENTINEL']['SENTINEL_USER']
API_SENTINEL_PASS = config['API_SENTINEL']['SENTINEL_PASS']
###########################################################

sys.path.append(SENTINEL_PATH)
sentinel = APISentinel.APISentinel(API_SENTINEL_USER, API_SENTINEL_PASS)


# Create your views here.
# -------------------------------------------------------------------------------
def SCLJSON2():
    return


# -------------------------------------------------------------------------------
# -------------------------------------------------------------------------------
def login(request):
    """
    View function for home page of site.
    """

    # Render the HTML template index.html with the data in the context variable
    return render(request, 'login.html', {})


# -------------------------------------------------------------------------------
def map(request):
    """
    View function for home page of site.
    """
    form = ASFSearchForm(request.POST)
    showcart = True
    # Render the HTML template index.html with the data in the context variable
    return render(request, 'map.html', {"searchForm": form, "showcart": showcart})


# -------------------------------------------------------------------------------
def requestToImage(request):
    img_link = request.GET['value']
    r = requests.get(img_link, auth=("emmhp", "geoemm29"), stream=True)
    img_str = base64.b64encode(r.content)
    img_str = img_str.decode('utf-8')

    return JsonResponse({'img': img_str})


# -------------------------------------------------------------------------------
def productList(request):
    """
    View function for home page of site.
    """
    req = dict(request.POST)
    req.pop('csrfmiddlewaretoken', None)

    user = User.objects.get(id=request.user.id)
    process = req['platform'][0]
    area = req['polygon'][0]
    init_date = req['start'][0][:10]
    end_date = req['end'][0][:10]
    clouds = req['cloudPercentage'][0]
    cloud_percentage = "[0 TO " + req['cloudPercentage'][0] + "]"

    ## ----- aqui llamada a api sentinel
    ## ----- antes consultar en DB si no se tiene ya el producto
    date = ("".join(init_date.split('-')), "".join(end_date.split('-')))
    params = {"platformname": "Sentinel-2"}

    # if cloud_percentage.strip():
    #     params['cloudcoverpercentage'] = cloud_percentage
    params['cloudcoverpercentage'] = cloud_percentage

    ## getting the bounding box of the polygon
    pol_bbox = polygonToBox.getWKTPolygonBoundingBox(area, True)

    ## requesting producto to sentinel api
    ##products = sentinel.getProducts(pol_bbox, date, params)
    products = sentinel.getProducts(pol_bbox, date,
                                    {"platformname": "Sentinel-2", "cloudcoverpercentage": cloud_percentage},
                                    "Contains")  ##intenta obtener los productos que contienen el polígono
    if len(products) <= 0:
        products = sentinel.getProducts(pol_bbox, date, {"platformname": "Sentinel-2",
                                                         "cloudcoverpercentage": cloud_percentage})  ## si no hay que contengan el polígono busca los que lo intersecan

    ## path from image preview storage location
    img_preview = '/static/images/sat_preview/sat_test.jpg'

    ## se crea una lista de objetos del catalogo
    catalog = []
    for p in products:
        # img_link = products[p]['link_icon']
        # img_link = img_link[:img_link.find("/")+2] + "emmhp:geoemm29@" + img_link[img_link.find("/")+2:]

        if products[p].get('tileid') is None:
            titleid = products[p]['title'].split("_")
            products[p]['tileid'] = titleid[5][1:]

        catalog.append({
            'process': process,
            'start_date': init_date,
            'end_date': end_date,
            'cloud_percentage': cloud_percentage,
            'product': products[p],
            'uuid': products[p]['uuid'],
            'img': img_preview
        })

    # Se tiene que pasar a Search el objeto completo correspondiente al process
    index = Process.objects.filter(platform_id=process)
    proceso = index[0]

    if (len(catalog) != 0):
        listSearch = Search(
            user=user,
            startDate=init_date,
            endDate=end_date,
            process=proceso,  # Aquí pasar el objeto completo de Process Desbloqueo cambio de Sergio
            clouds=clouds,
            area=area
        )
        listSearch.save()

    return JsonResponse({'catalog': catalog});
    # return HttpResponse(status=204)


# -------------------------------------------------------------------------------
class SearchSubmitView(View):
    template = 'map.html'

    def strip_accents(self, s):
        return ''.join((c for c in unicodedata.normalize('NFD', s) if unicodedata.category(c) != 'Mn'))

    def post(self, request):
        template = loader.get_template(self.template)
        searchValue = request.POST.get('value', '')
        searchValue = self.strip_accents(searchValue).lower()

        ## A simple query for Polygon objects whose name is 'searchValue'
        ## We exclude those polygons whose E_MUN field are NULL .encode("utf-8")since at this point we're only interested
        ## on polygons that represent cities (Municipios).
        polygonList = Polygon.objects.filter(name__startswith=searchValue).exclude(E_MUN__isnull=True)

        data_list = []
        if len(polygonList) > 0:
            for polygon in polygonList:
                polygonInfo = json.loads(polygon.json_info)

                data_list.append({
                    'id': str(polygon.id),
                    'city': polygonInfo['properties']['NOMGEO'],
                    'state': polygonInfo['properties']['ENT_NAME'],
                    'geojson': polygonInfo,
                    'wkt_polygon': polygon.wkt_polygon,
                    'description': polygon.description,
                    'source': polygon.source
                })

        data = {
            'polygonList': data_list
        }

        return JsonResponse(data)


# -------------------------------------------------------------------------------
def regionSearched(request):
    """
    View function for searching regions in DB
    """

    city_name = request.POST['value']
    cityList = Polygon.objects.filter(name=city_name).exclude(E_MUN__isnull=True)

    if len(cityList) > 0:
        print(cityList)

    return HttpResponseRedirect(reverse('map'))


# -------------------------------------------------------------------------------
def saveInCart(request):
    """
    Saves product in cart table in DB
    """

    user = User.objects.get(id=request.user.id)
    product_list = json.loads(request.POST['cart_product_list'])
    search = Search.objects.filter(user=user).last()

    if (len(product_list) != 0):
        cartProd = Purchase(
            user=user,
            search=search,
            productList=json.dumps(product_list, indent=3, sort_keys=True, ensure_ascii=True),
            purchased=False,
            price=5.0
        )
        cartProd.save()

    return HttpResponse(status=204)


# -------------------------------------------------------------------------------
def getFromCart(request):
    """
    Temporarily gets product from cart table based on uuid porperty to show in cart section
    """

    user = User.objects.get(id=request.user.id)
    prod_list = []
    product_list = Purchase.objects.values('user', 'productList', 'purchased', 'price', 'aggreg_date', 'id',
                                           'search__process__name', 'search__startDate', 'search__endDate').filter(
        user=user, purchased=0)

    for products in product_list:
        product = json.loads(products['productList'])
        prod_list.append({
            'purchased': products['purchased'],
            'price': products['price'],
            'aggreg_date': products['aggreg_date'],
            'id': products['id'],
            'process': products['search__process__name'],
            'startDate': products['search__startDate'],
            'endDate': products['search__endDate'],
            'catalog': product
        })
    return JsonResponse({'product_list': prod_list})


# -------------------------------------------------------------------------------
def delFromCart(request):
    user = User.objects.get(id=request.user.id)
    product_list = Purchase.objects.filter(user=user, id=request.POST['id']).delete()
    return HttpResponse(status=204)


##-------------------------------------------------------------------------------
# def L2ASCL_data(request):
#	with open("/home/mario/NAS_MarioCh/mario_repsat_test/T15QZD_sclData.json") as f:
#		json_data = OrderedDict(json.load(f))
#		graphData = dict()
#		graphData['type']="line"
#		data=dict()
#		data['labels'] = [ k for k,v in sorted(json_data.items()) ]

#		data['datasets'] = list()
#		labels  = set()
#		for d in json_data:
#			labels |= set(json_data[d].keys())
#		labels.remove('date')

#		labelNames = ["No Data", "Saturated", "Shadows", "Cloud Shadows", "Vegetation",
#                   "Not-Vegetated", "Whater", "Unclassified", "Cloud Medium Probablilty", "Cloud Hight Probablilty", "Thin Cirrus", "Snow"]
#		colors = ["Black", "Red", "DarkGray", "Brown", "Green", "Yellow", "Blue", "Grey", "LightGray", "White", "Aqua", "Pink"]
#		for l in labels:
#			item = dict()
#			item['label'] = labelNames[int(l)]
#			if l == "0":
#				item['hidden'] = "true"
#			item['data'] = [ v[l] * 60*60/1000000. for k,v in sorted(json_data.items()) if l in v]
#			item['backgroundColor']=colors[int(l)]
#			data['datasets'].append(item)

#		graphData['data']=data

##		graphData['options']['responsive']=True
##		graphData['options']['maintainAspectRatio']=False
#		print( json.dumps(graphData).encode("utf-8"))
#	return render(request, 'L2ASCL_data.html', {"graphData":json.dumps(graphData).encode("utf-8")})
# -------------------------------------------------------------------------------
def wsReportImages(request):
    print(request.GET)


# -------------------------------------------------------------------------------
def purchaseProduct(request):
    """
    For now to purchase a product consists in verifying whether L1C products in cart already exist in DB.
    """
    # ------------- crear archivo findProducts.json

    print("Comprar productos!!!!")

    user = User.objects.get(id=request.user.id)
    prod_list = []
    product_list = Search.objects.values('user', 'area', 'startDate', 'endDate', 'clouds').filter(user=user,
                                                                                                  purchased=0)

    data = {}

    for products in product_list:
        print(products)
        # ~ product = json.loads(products['productList'])
        # ~ prod_list.append({
        # ~ 'purchased': products['purchased'],
        # ~ 'price'  : products['price'],
        # ~ 'aggreg_date' : products['aggreg_date'],
        # ~ 'catalog': product
        # ~ })
        # ~ return JsonResponse({'product_list': prod_list})



        # ~ print("Comprar productos!!!!")
        # ~ print(request)
        # ~ print("start date:")
        # ~ print(request.POST['wkt'])
        # ~ print(request.POST['startDate'])
        # ~ print(request.POST['endDate'])
        # ~ print(request.POST['clouds'])
        # ~ print(request.POST['usuario'])

        # ~ username = request.POST['usuario']
        # ~ fecha_inicio = request.POST['startDate']
        # ~ fecha_fin = request.POST['endDate']

        # ~ print(fecha_inicio.replace("-", ""))
        # ~ print(fecha_fin.replace("-", ""))

        # ~ userdir = NAS_PATH + "repsat_test_dev/"+username   ##crea el directorio del usuario
        # ~ if not os.path.exists(userdir):
        # ~ os.makedirs(userdir)

    # ~ products_dir = NAS_PATH + "sentinelImages/L2A/"
    # ~ linksDir = userdir+"/L2A/"

    # ~ #obtiene los datos para el json
    # ~ data = {}
    # ~ data['wkt'] = request.POST['wkt']
    # ~ data['platform'] = 'Sentinel-2'
    # ~ data['productLevel'] = 'L2A'
    # ~ data['startDate'] = fecha_inicio.replace("-", "")  #quita los guiones de la fecha
    # ~ data['endDate'] = fecha_fin.replace("-", "")       #quita los guiones de la fecha
    # ~ # data['startDate'] = request.POST['startDate']
    # ~ # data['endDate'] = request.POST['endDate']
    # ~ data['clouds'] = request.POST['clouds']
    # ~ # data['productsDir'] = '/home/david/NAS/sentinelImages/L2A/'
    # ~ # data['linksDir'] = '/home/david/centroGEO/repsat/tests/L2A/'
    # ~ data['productsDir'] = products_dir
    # ~ data['linksDir'] = linksDir
    # ~ data['username'] = 'mario-chirinos'
    # ~ data['password'] = 'r4nc0r4u'

    # ~ #json_data = json.dumps(data, indent=4,sort_keys=True)
    # ~ #formatea el json
    # ~ json_data = json.dumps(data, indent=4)
    # ~ print(json_data)
    # ~ #escribe el json en el archivo
    # ~ fidProductsFile = open(userdir+"/findProducts.json", "w")
    # ~ fidProductsFile.write(json_data)
    # ~ fidProductsFile.close()

    # -------------------------------------------------

    products = json.loads(request.POST['product_objects'])
    orderedProducts = OrderedDict()

    for p_uuid in products.keys():
        PRODUCT_EXISTS = Product_l1c.objects.filter(uuid=p_uuid).exists()

        if not PRODUCT_EXISTS:
            orderedProducts[p_uuid] = products[p_uuid]

    print(orderedProducts)
    return HttpResponse(status=204)

    ## TODO: Implement a webservice in server for downloading those non-existing L1C products. ##
    ## TODO: Add product L1C to Cart DB after product download. ##


# -------------------------------------------------------------------------------
# def L2ASCL_data(request):
#     with open("/home/ulises/REPSAT_NAS/mario_repsat_test/T15QZD_sclData.json") as f:
#     	json_data = json.load(f)
#     	print(json_data)
#     return render(request, 'L2ASCL_data.html', {"labels":",".join([k for k in json_data]), "vegetation":[v["4"] for k,v in json_data.items()] })
# -------------------------------------------------------------------------------
# def purchaseProduct(request):
#     """
#     For now to purchase a product consists in downloading L1C products and saving them in DB.
#     """
#     ################# DEFINE PATH ##########################
#     prod_dir             = 'my_path'
#     ########################################################
#     products_to_download = {
#         'uuid' : [],
#         'id'   : []
#     }

#     user             = User.objects.get(id=request.user.id)
#     prod_purchased = json.loads(request.POST['product_list'])
#     # prod_purchased   = ['S2B_MSIL1C_20171226T162659_N0206_R040_T16QBJ_20171226T212212', 'S2B_MSIL1C_20180326T161909_N0206_R040_T15QZD_20180326T195757']

#     for p in prod_purchased:
#         p_uuid         = p['uuid']
#         p_identifier   = p['identifier']
#         PRODUCT_EXISTS = Product_l1c.objects.filter(uuid=p_uuid).exists()

#         if not PRODUCT_EXISTS:
#             products_to_download['uuid'].append(p_uuid)
#             products_to_download['id'].append(p_identifier)

#         # print json.dumps(p, indent=3, sort_keys=True)

#     ## Saves data to DB using populate_products_l1c command ##
#     management.call_command('populate_products_l1c', file_path=prod_dir, products_list=products_to_download['id'])

#     ## Register of user purchase ##
#     purch = Purchase(user=user)
#     purch.save()

#     for p_uuid in products_to_download['uuid']:
#         prod_l1c = Product_l1c.objects.get(uuid=p_uuid)
#         purch.prod_list.add(prod_l1c)





#     ### TODO: Download products somewhere .. ###
#     # sentinel.downloadProducts(products, prod_dir)


#     ###  After that the code below is executed  ###
#     # purch = Purchase(user=user)
#     # purch.save()

#     # # for prod_uuid in prod_purchased:
#     # for prod_id in prod_purchased:
#     #     # prod_l1c = Product_l1c.objects.get(uuid=prod_uuid)
#     #     prod_l1c = Product_l1c.objects.get(identifier=prod_id)
#     #     print prod_l1c
#     #     purch.prod_list.add(prod_l1c)


#    return HttpResponse(status=204)

# -------------------------------------------------------------------------------
def showCart(request):
    user = User.objects.get(id=request.user.id)
    prod_list = []
    product_list = Purchase.objects.values('user', 'productList', 'purchased', 'price', 'aggreg_date', 'id',
                                           'search__process_id__name').filter(user=user, purchased=0)
    total_price = 0
    total_products = 0

    for products in product_list:
        product = json.loads(products['productList'])
        prod_list.append({
            'purchased': products['purchased'],
            'price': products['price'],
            'aggreg_date': products['aggreg_date'],
            'catalog': product,
            'id': products['id'],
            "process": products['search__process_id__name'],
        })
        total_price += products['price']
        total_products += 1

    return render(request, 'show_cart.html',
                  {'product_list': prod_list, 'total_price': total_price, 'total_products': total_products})

    # -------------------------------------


def Pay(request):
    # ---------------test process----------------------
    print("PAGAR!!!!")
    user = User.objects.get(id=request.user.id)
    product_list = []
    product_list = Purchase.objects.values('id', 'search__area', 'user', 'user__username', 'user__email', 'search_id',
                                           'search__clouds', 'search__startDate', 'search__endDate').filter(user=user,
                                                                                                            purchased=0)

    for product in product_list:

        # print(product['user__username'])
        # print(product['user__email'])
        # print(product['search_id'])
        # print(str(product['search__startDate'].date()).replace("-", ""))
        # print(str(product['search__endDate'].date()).replace("-", ""))
        # print(product['search__clouds'])
        # print(product['search__area'])
        print(product['id'])
        username = product['user__username']
        useremail = product['user__email']

        # userdir = NAS_PATH + "repsat_test_dev/"+useremail   ##crea el directorio del usuario
        userdir = USERS_PATH + useremail

        if not os.path.exists(userdir):
            os.makedirs(userdir)

        purchase_dir = userdir + "/" + str(product['id'])
        if not os.path.exists(purchase_dir):
            os.makedirs(purchase_dir)

        products_dir = NAS_PATH + "sentinelImages/L2A/"
        linksDir = purchase_dir + "/L2A/"

        # obtiene los datos para el json
        data = {}
        data['wkt'] = product['search__area']
        data['platform'] = 'Sentinel-2'
        data['productLevel'] = 'L2A'
        data['startDate'] = str(product['search__startDate'].date()).replace("-", "")  # quita los guiones de la fecha
        data['endDate'] = str(product['search__endDate'].date()).replace("-", "")  # quita los guiones de la fecha
        data['clouds'] = product['search__clouds']
        data['productsDir'] = products_dir
        data['linksDir'] = linksDir
        data['username'] = API_SENTINEL_USER
        data['password'] = API_SENTINEL_PASS

        # json_data = json.dumps(data, indent=4,sort_keys=True)
        # formatea el json
        json_data = json.dumps(data, indent=4)
        # print(json_data)
        # escribe el json en el archivo
        fidProductsFile = open(purchase_dir + "/findProducts.json", "w")
        fidProductsFile.write(json_data)
        fidProductsFile.close()

        os.system(
            SENTINEL_PATH + "/tools/L2ASCL_AreaProcessing.sh " + purchase_dir + "/ 1 > " + purchase_dir + "/L2ASCL_AreaProcessing.log")  ###ejecuta el proceso



        # ~ user           = User.objects.get(id=request.user.id)
        # ~ prod_list      = []
        # ~ product_list = Search.objects.values('user', 'area', 'startDate', 'endDate','clouds').filter(user=user)

        # ~ data = {}

        # ~ for products in product_list:
        # ~ print(products)
        # ~ product = json.loads(products['productList'])
        # ~ prod_list.append({
        # ~ 'purchased': products['purchased'],
        # ~ 'price'  : products['price'],
        # ~ 'aggreg_date' : products['aggreg_date'],
        # ~ 'catalog': product
        # ~ })
    # ~ return JsonResponse({'product_list': prod_list})

    # -------------------------------------

    Purchase.objects.filter(user_id=request.user.id, purchased=0).update(purchased=True)
    return redirect('../../reports')


# -----------------------------------------------------------------
def IPN(request):
    print("content-type: text/html\r\n\r\n")

    if request.POST['payment_status'] == 'Completed':
        Purchase.objects.filter(user_id=request.user.id, purchased=0).update(purchased=True)
        return redirect('../../reports')
    else:
        return redirect('../cart')
        # f.write(request.GET)

        # f.close()
        # PAYPAL_URL = "https://sandbox.paypal.com/cgi-bin/webscr"
        #
        # formData = "cmd=_notify-validate&" + parse.unquote(inputraw)
        #
        # f.write(formData);
        # f.write("\n")  # write into file 1st data package from paypal
        #
        # req = urllib.request.Request(PAYPAL_URL, formData.encode())
        # req.add_header("Content-type", "application/x-www-form-urlencoded")
        # response = urllib.request.urlopen(req)
        # status = str(response.read())
        #
        # f.write(status)  # write back into file what you received from paypal the 2nd time
        # f.write("\n")
        #
        # if (not status == "b'VERIFIED'"):
        #     f.write("OH NO")
        #     # ERROR
        # else:
        #     # HERE WE WILL TRIGGER STUFF WITH THE DATA FROM PAYPAL
        #     f.write("OH YES")
        #
        # f.write("\n")
        # f.close()


# -----------------------------------------------------------------------
# def Pay3(request):
#     VERIFY_URL_PROD = 'https://ipnpb.paypal.com/cgi-bin/webscr'
#     VERIFY_URL_TEST = 'https://ipnpb.sandbox.paypal.com/cgi-bin/webscr'
#
#     # Switch as appropriate
#     VERIFY_URL = VERIFY_URL_TEST
#
#     # CGI preamble
#     print('content-type: text/plain')
#     print()
#
#     # Read and parse query string
#     param_str = sys.stdin.readline().strip()
#     params = urllib.parse.parse_qsl(param_str)
#
#     # Add '_notify-validate' parameter
#     params.append(('cmd', '_notify-validate'))
#
#     # Post back to PayPal for validation
#
#     headers = {'content-type': 'application/x-www-form-urlencoded',
#                'user-agent': 'Python-IPN-Verification-Script'}
#     r = requests.post(VERIFY_URL, params=params, headers=headers, verify=True)
#     r.raise_for_status()
#
#     # Check return message and take action as needed
#     if r.text == 'VERIFIED':
#         pass
#     elif r.text == 'INVALID':
#         pass
#     else:
#         pass
#     return redirect("https://google.com.mx")
# ------------------------------------------------------
def Registration(request):
    #user = User.objects.filter(email=request.POST['email'])
    if User.objects.filter(email=request.POST['email']):
        print("ya existe")
        return HttpResponse("el correo ya está registrado")
    else:
        print("agregado")
        User.objects.create_user(username=request.POST['username'],
                             email=request.POST['email'],
                             password=request.POST['password'], is_active=False)
        message = "your registration is being processed"
        subject = "Registro RepSat"
        wsMail2(request.POST['email'],request.POST['username'],message,subject)
        messages.info(request, 'your registration is being processed')
        return HttpResponseRedirect('login')
